@@ -103,6 +103,12 @@ INVALID_NC_FLAG_CONFIG_OVERLAP = (
     'or the value in the config file.'
 )
 
+INVALID_NC_KERNEL_MODULE_LOADING_FLAG_CONFIG_OVERLAP = (
+    'enable-kernel-module-signature-enforcement specified in both config '
+    'file and by flag. Please specify either command line option '
+    'or the value in the config file.'
+)
+
 
 GKE_DEFAULT_POD_RANGE = 14
 GKE_DEFAULT_POD_RANGE_PER_NODE = 24
@@ -144,6 +150,10 @@ NC_KERNEL_INIT_ON_ALLOC = 'initOnAlloc'
 NC_LRU_GEN = 'lruGen'
 NC_LRU_GEN_ENABLED = 'enabled'
 NC_LRU_GEN_MIN_TTL_MS = 'minTtlMs'
+NC_SHUTDOWN_GRACE_PERIOD_SECONDS = 'shutdownGracePeriodSeconds'
+NC_SHUTDOWN_GRACE_PERIOD_CRITICAL_PODS_SECONDS = (
+    'shutdownGracePeriodCriticalPodsSeconds'
+)
 
 NC_ADDITIONAL_ETC_HOSTS = 'additionalEtcHosts'
 NC_ADDITIONAL_ETC_SYSTEMD_RESOLVED_CONF = 'additionalEtcSystemdResolvedConf'
@@ -154,6 +164,9 @@ NC_CUSTOM_NODE_INIT_SCRIPT = 'initScript'
 NC_CUSTOM_NODE_INIT_SCRIPT_GCS_URI = 'gcsUri'
 NC_CUSTOM_NODE_INIT_SCRIPT_GCS_GENERATION = 'gcsGeneration'
 NC_CUSTOM_NODE_INIT_SCRIPT_ARGS = 'args'
+NC_CUSTOM_NODE_INIT_SCRIPT_GCP_SECRET_MANAGER_SECRET_URI = (
+    'gcpSecretManagerSecretUri'
+)
 NC_ETC_HOSTS_ENTRY_IP = 'ip'
 NC_ETC_HOSTS_ENTRY_HOST = 'host'
 NC_RESOLVED_CONF_ENTRY_KEY = 'key'
@@ -173,6 +186,25 @@ NC_CC_PRIVATE_CR_CONFIG = 'privateRegistryAccessConfig'
 NC_CC_PRIVATE_CR_CONFIG_ENABLED = 'enabled'
 NC_CC_WRITABLE_CGROUPS = 'writableCgroups'
 NC_CC_WRITABLE_CGROUPS_ENABLED = 'enabled'
+
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS = 'registryHosts'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_SERVER = 'server'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HOSTS = 'hosts'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HOST = 'host'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_OVERRIDE_PATH = 'overridePath'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HEADER = 'header'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HEADER_KEY = 'key'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HEADER_VALUE = 'value'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_DIAL_TIMEOUT = 'dialTimeout'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CAPABILITIES = 'capabilities'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_GCP_SECRET_URI = (
+    'gcpSecretManagerSecretUri'
+)
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CA = 'ca'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CLIENT = 'client'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CLIENT_CERT = 'cert'
+NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CLIENT_KEY = 'key'
+
 NC_CC_CA_CONFIG = 'certificateAuthorityDomainConfig'
 NC_CC_GCP_SECRET_CONFIG = 'gcpSecretManagerCertificateConfig'
 NC_CC_GCP_SECRET_CONFIG_SECRET_URI = 'secretURI'
@@ -183,6 +215,9 @@ NC_EVICTION_SOFT_GRACE_PERIOD = 'evictionSoftGracePeriod'
 NC_EVICTION_MINIMUM_RECLAIM = 'evictionMinimumReclaim'
 NC_EVICTION_MAX_POD_GRACE_PERIOD_SECONDS = 'evictionMaxPodGracePeriodSeconds'
 
+NC_KERNEL_MODULE_LOADING = 'nodeKernelModuleLoading'
+NC_KERNEL_MODULE_LOADING_POLICY = 'policy'
+
 
 class Error(core_exceptions.Error):
   """Class for errors raised by container commands."""
@@ -837,7 +872,11 @@ def LoadEvictionMapConfig(parent_name, opts, msg_type, field_spec):
 
 
 def LoadSystemConfigFromYAML(
-    node_config, content, opt_readonly_port_flag, messages
+    node_config,
+    content,
+    opt_readonly_port_flag,
+    opt_enable_kernel_module_signature_enforcement,
+    messages
 ):
   """Load system configuration (sysctl & kubelet config) from YAML/JSON file.
 
@@ -845,6 +884,8 @@ def LoadSystemConfigFromYAML(
     node_config: The node config object to be populated.
     content: The YAML/JSON string that contains sysctl and kubelet options.
     opt_readonly_port_flag: kubelet readonly port enabled.
+    opt_enable_kernel_module_signature_enforcement: enable kernel module
+      signature enforcement.
     messages: The message module.
 
   Raises:
@@ -886,6 +927,8 @@ def LoadSystemConfigFromYAML(
         NC_SINGLE_PROCESS_OOMKILL: bool,
         NC_NODE_SWAP_SIZE_GIB: int,
         NC_MAX_PARALLEL_IMAGE_PULLS: int,
+        NC_SHUTDOWN_GRACE_PERIOD_SECONDS: int,
+        NC_SHUTDOWN_GRACE_PERIOD_CRITICAL_PODS_SECONDS: int,
         NC_EVICTION_SOFT: dict,
         NC_EVICTION_SOFT_GRACE_PERIOD: dict,
         NC_EVICTION_MINIMUM_RECLAIM: dict,
@@ -937,6 +980,14 @@ def LoadSystemConfigFromYAML(
     node_config.kubeletConfig.maxParallelImagePulls = kubelet_config_opts.get(
         NC_MAX_PARALLEL_IMAGE_PULLS
     )
+    node_config.kubeletConfig.shutdownGracePeriodSeconds = (
+        kubelet_config_opts.get(NC_SHUTDOWN_GRACE_PERIOD_SECONDS)
+    )
+    node_config.kubeletConfig.shutdownGracePeriodCriticalPodsSeconds = (
+        kubelet_config_opts.get(
+            NC_SHUTDOWN_GRACE_PERIOD_CRITICAL_PODS_SECONDS
+        )
+    )
 
     # Populate eviction fields
     eviction_map_string_fields = {
@@ -1027,6 +1078,7 @@ def LoadSystemConfigFromYAML(
             NC_ADDITIONAL_ETC_RESOLV_CONF: list,
             NC_TIME_ZONE: str,
             NC_CUSTOM_NODE_INIT: dict,
+            NC_KERNEL_MODULE_LOADING: dict,
         },
     )
     node_config.linuxNodeConfig = messages.LinuxNodeConfig()
@@ -1422,11 +1474,27 @@ def LoadSystemConfigFromYAML(
                 NC_CUSTOM_NODE_INIT_SCRIPT_GCS_URI: str,
                 NC_CUSTOM_NODE_INIT_SCRIPT_GCS_GENERATION: int,
                 NC_CUSTOM_NODE_INIT_SCRIPT_ARGS: list,
+                NC_CUSTOM_NODE_INIT_SCRIPT_GCP_SECRET_MANAGER_SECRET_URI: str,
             },
         )
-        node_config.linuxNodeConfig.customNodeInit.initScript.gcsUri = (
-            init_script_opts.get(NC_CUSTOM_NODE_INIT_SCRIPT_GCS_URI)
+        gcs_uri = init_script_opts.get(NC_CUSTOM_NODE_INIT_SCRIPT_GCS_URI)
+        gcp_secret_manager_secret_uri = init_script_opts.get(
+            NC_CUSTOM_NODE_INIT_SCRIPT_GCP_SECRET_MANAGER_SECRET_URI
         )
+
+        if gcs_uri and gcp_secret_manager_secret_uri:
+          raise NodeConfigError(
+              f'Cannot specify both [{NC_CUSTOM_NODE_INIT_SCRIPT_GCS_URI}] and'
+              f' [{NC_CUSTOM_NODE_INIT_SCRIPT_GCP_SECRET_MANAGER_SECRET_URI}]'
+              ' for custom node init script.'
+          )
+        if gcs_uri:
+          node_config.linuxNodeConfig.customNodeInit.initScript.gcsUri = gcs_uri
+        if gcp_secret_manager_secret_uri:
+          node_config.linuxNodeConfig.customNodeInit.initScript.gcpSecretManagerSecretUri = (
+              gcp_secret_manager_secret_uri
+          )
+
         node_config.linuxNodeConfig.customNodeInit.initScript.gcsGeneration = (
             init_script_opts.get(NC_CUSTOM_NODE_INIT_SCRIPT_GCS_GENERATION)
         )
@@ -1434,6 +1502,45 @@ def LoadSystemConfigFromYAML(
         if init_args is not None:
           node_config.linuxNodeConfig.customNodeInit.initScript.args = init_args
 
+    kernel_module_loading_opts = linux_config_opts.get(NC_KERNEL_MODULE_LOADING)
+    if kernel_module_loading_opts:
+      module_loading_config = messages.NodeKernelModuleLoading()
+      _CheckNodeConfigFields(
+          NC_KERNEL_MODULE_LOADING,
+          kernel_module_loading_opts,
+          {
+              NC_KERNEL_MODULE_LOADING_POLICY: str,
+          },
+      )
+      policy = kernel_module_loading_opts.get(NC_KERNEL_MODULE_LOADING_POLICY)
+      if policy is not None and opt_enable_kernel_module_signature_enforcement:
+        raise NodeConfigError(
+            INVALID_NC_KERNEL_MODULE_LOADING_FLAG_CONFIG_OVERLAP
+        )
+      if policy is not None:
+        policy_enum = messages.NodeKernelModuleLoading.PolicyValueValuesEnum
+        kernel_module_loading_policy_mapping = {
+            'ENFORCE_SIGNED_MODULES': policy_enum.ENFORCE_SIGNED_MODULES,
+            'DO_NOT_ENFORCE_SIGNED_MODULES': (
+                policy_enum.DO_NOT_ENFORCE_SIGNED_MODULES
+            ),
+            'POLICY_UNSPECIFIED': policy_enum.POLICY_UNSPECIFIED,
+        }
+        if policy not in kernel_module_loading_policy_mapping:
+          raise NodeConfigError(
+              'kernel module loading policy "{0}" is not supported, the'
+              ' supported options are ENFORCE_SIGNED_MODULES,'
+              ' DO_NOT_ENFORCE_SIGNED_MODULES, POLICY_UNSPECIFIED'.format(
+                  policy
+              )
+          )
+        module_loading_config.policy = kernel_module_loading_policy_mapping[
+            policy
+        ]
+        node_config.linuxNodeConfig.nodeKernelModuleLoading = (
+            module_loading_config
+        )
+
 
 def CheckForCgroupModeV1(pool):
   """Check cgroup mode of the node pool and print a warning if it is V1."""
@@ -1468,6 +1575,7 @@ def LoadContainerdConfigFromYAML(containerd_config, content, messages):
       {
           NC_CC_PRIVATE_CR_CONFIG: dict,
           NC_CC_WRITABLE_CGROUPS: dict,
+          NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS: list,
       },
   )
 
@@ -1534,6 +1642,187 @@ def LoadContainerdConfigFromYAML(containerd_config, content, messages):
         NC_CC_WRITABLE_CGROUPS_ENABLED
     )
 
+  registry_hosts_opts = opts.get(NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS)
+  if registry_hosts_opts:
+    containerd_config.registryHosts = []
+    for i, registry_host_opt in enumerate(registry_hosts_opts):
+      _CheckNodeConfigFields(
+          '{0}[{1}]'.format(NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS, i),
+          registry_host_opt,
+          {
+              NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_SERVER: str,
+              NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HOSTS: list,
+          },
+      )
+      registry_host_msg = messages.RegistryHostConfig()
+      registry_host_msg.server = registry_host_opt.get(
+          NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_SERVER
+      )
+
+      hosts_opts = registry_host_opt.get(
+          NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HOSTS
+      )
+      if hosts_opts:
+        registry_host_msg.hosts = []
+        for j, host_opt in enumerate(hosts_opts):
+          _CheckNodeConfigFields(
+              '{0}[{1}].{2}[{3}]'.format(
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS,
+                  i,
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HOSTS,
+                  j,
+              ),
+              host_opt,
+              {
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HOST: str,
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_OVERRIDE_PATH: bool,
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CAPABILITIES: list,
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HEADER: list,
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_DIAL_TIMEOUT: str,
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CA: list,
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CLIENT: list,
+              },
+          )
+          host_msg = messages.HostConfig()
+          host_msg.host = host_opt.get(
+              NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HOST
+          )
+          host_msg.overridePath = host_opt.get(
+              NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_OVERRIDE_PATH
+          )
+
+          capabilities_opts = host_opt.get(
+              NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CAPABILITIES
+          )
+          if capabilities_opts:
+            capability_mapping = {
+                'HOST_CAPABILITY_UNSPECIFIED': (
+                    messages.HostConfig.CapabilitiesValueListEntryValuesEnum.HOST_CAPABILITY_UNSPECIFIED
+                ),
+                'HOST_CAPABILITY_PULL': (
+                    messages.HostConfig.CapabilitiesValueListEntryValuesEnum.HOST_CAPABILITY_PULL
+                ),
+                'HOST_CAPABILITY_RESOLVE': (
+                    messages.HostConfig.CapabilitiesValueListEntryValuesEnum.HOST_CAPABILITY_RESOLVE
+                ),
+                'HOST_CAPABILITY_PUSH': (
+                    messages.HostConfig.CapabilitiesValueListEntryValuesEnum.HOST_CAPABILITY_PUSH
+                ),
+            }
+            for cap_opt in capabilities_opts:
+              if cap_opt not in capability_mapping:
+                raise NodeConfigError(
+                    'capability "{0}" is not supported, the supported options'
+                    ' are HOST_CAPABILITY_PULL, HOST_CAPABILITY_RESOLVE,'
+                    ' HOST_CAPABILITY_PUSH'.format(cap_opt)
+                )
+              if not host_msg.capabilities:
+                host_msg.capabilities = []
+              host_msg.capabilities.append(capability_mapping[cap_opt])
+
+          header_opts = host_opt.get(
+              NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HEADER
+          )
+          if header_opts:
+            host_msg.header = []
+            for header_opt in header_opts:
+              _CheckNodeConfigFields(
+                  'header',
+                  header_opt,
+                  {
+                      NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HEADER_KEY: str,
+                      NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HEADER_VALUE: list,
+                  },
+              )
+              header_msg = messages.RegistryHeader()
+              header_msg.key = header_opt.get(
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HEADER_KEY
+              )
+              header_msg.value = header_opt.get(
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_HEADER_VALUE
+              )
+              host_msg.header.append(header_msg)
+
+          dial_timeout_opt = host_opt.get(
+              NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_DIAL_TIMEOUT
+          )
+          if dial_timeout_opt:
+            host_msg.dialTimeout = dial_timeout_opt
+
+          ca_opts = host_opt.get(NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CA)
+          if ca_opts:
+            host_msg.ca = []
+            for ca_opt in ca_opts:
+              _CheckNodeConfigFields(
+                  'ca',
+                  ca_opt,
+                  {NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_GCP_SECRET_URI: str},
+              )
+              ca_msg = messages.CertificateConfig()
+              ca_msg.gcpSecretManagerSecretUri = ca_opt.get(
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_GCP_SECRET_URI
+              )
+              host_msg.ca.append(ca_msg)
+
+          client_opts = host_opt.get(
+              NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CLIENT
+          )
+          if client_opts:
+            host_msg.client = []
+            for client_opt in client_opts:
+              _CheckNodeConfigFields(
+                  'client',
+                  client_opt,
+                  {
+                      NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CLIENT_CERT: dict,
+                      NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CLIENT_KEY: dict,
+                  },
+              )
+              client_msg = messages.CertificateConfigPair()
+
+              cert_opt = client_opt.get(
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CLIENT_CERT
+              )
+              if cert_opt:
+                _CheckNodeConfigFields(
+                    'cert',
+                    cert_opt,
+                    {
+                        NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_GCP_SECRET_URI: (
+                            str
+                        )
+                    },
+                )
+                cert_msg = messages.CertificateConfig()
+                cert_msg.gcpSecretManagerSecretUri = cert_opt.get(
+                    NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_GCP_SECRET_URI
+                )
+                client_msg.cert = cert_msg
+
+              key_opt = client_opt.get(
+                  NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_CLIENT_KEY
+              )
+              if key_opt:
+                _CheckNodeConfigFields(
+                    'key',
+                    key_opt,
+                    {
+                        NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_GCP_SECRET_URI: (
+                            str
+                        )
+                    },
+                )
+                key_msg = messages.CertificateConfig()
+                key_msg.gcpSecretManagerSecretUri = key_opt.get(
+                    NC_CC_PRIVATE_CR_CONFIG_REGISTRY_HOSTS_GCP_SECRET_URI
+                )
+                client_msg.key = key_msg
+
+              host_msg.client.append(client_msg)
+
+          registry_host_msg.hosts.append(host_msg)
+      containerd_config.registryHosts.append(registry_host_msg)
+
 
 def _CheckNodeConfigFields(parent_name, parent, spec):
   """Check whether the children of the config option are valid or not.
@@ -2017,7 +2306,7 @@ def _GenerateIamToken() -> str:
         'Google account.'
     )
 
-  cred = c_store.Load(use_google_auth=True, allow_account_impersonation=False)
+  cred = c_store.Load(allow_account_impersonation=False)
   auth_token_file = properties.VALUES.auth.authorization_token_file.Get(
       required=True
   )
