@@ -50,6 +50,8 @@ class FeatureCommand(hub_base.HubCommand):
     return super(FeatureCommand,
                  self).FeatureResourceName(self.feature_name, project)
 
+  # TODO(b/440616932): Create custom class for this error so that users can
+  # check for this error by type.
   def FeatureNotEnabledError(self, project=None):
     """Constructs a new Error for reporting when this Feature is not enabled."""
     project = project or properties.VALUES.core.project.GetOrFail()
@@ -76,7 +78,17 @@ class FeatureCommand(hub_base.HubCommand):
 class EnableCommandMixin(FeatureCommand):
   """A mixin for functionality to enable a Feature."""
 
-  def Enable(self, feature):
+  def Enable(self, feature, error_if_feature_exists=False):
+    """Enables feature API and creates the feature.
+
+    Args:
+      feature: Fleet Feature to create.
+      error_if_feature_exists: Whether to error if feature already exists.
+        Example usage if caller populates feature fields that may differ on
+        existing Feature.
+    Returns:
+      Feature creation operation.
+    """
     project = properties.VALUES.core.project.GetOrFail()
     if self.feature.api:
       enable_api.EnableServiceIfDisabled(project, self.feature.api)
@@ -95,9 +107,9 @@ class EnableCommandMixin(FeatureCommand):
           'Retry limit exceeded waiting for {} to enable'.format(
               self.feature.display_name))
     except apitools_exceptions.HttpConflictError as e:
-      # If the error is not due to the object already existing, re-raise.
       error = core_api_exceptions.HttpErrorPayload(e)
-      if error.status_description != 'ALREADY_EXISTS':
+      if (error.status_description != 'ALREADY_EXISTS'
+          or error_if_feature_exists):
         raise
       log.status.Print('{} Feature for project [{}] is already enabled'.format(
           self.feature.display_name, project))
@@ -222,6 +234,37 @@ class UpdateCommand(UpdateCommandMixin, calliope_base.UpdateCommand):
   Because Features updates are often bespoke actions, there is no default
   `Run` override like some of the other classes.
   """
+  ORIGIN_FLAG = calliope_base.Argument(
+      '--origin',
+      type=lambda origin: origin.lower(),
+      choices=['fleet'],
+      help=(
+          'Update the configuration of the target membership(s) to the'
+          ' [fleet-default membership configuration]('
+          'https://docs.cloud.google.com/kubernetes-engine/fleet-management/docs/manage-features).'
+          ' Errors if the fleet-default membership configuration is not set;'
+          ' see the `--fleet-default-member-config` flag.'
+      ),
+  )
+
+  def sync_memberships_to_fleet_default(self, memberships):
+    # TODO(b/471048574): Switch to v2 API if it ever supports syncing to FDC.
+    membership_specs = self.messages.Feature.MembershipSpecsValue(
+        additionalProperties=[
+            self.messages.Feature.MembershipSpecsValue.AdditionalProperty(
+                key=m,
+                value=self.messages.MembershipFeatureSpec(
+                    origin=self.messages.Origin(
+                        type=self.messages.Origin.TypeValueValuesEnum.FLEET,
+                    )
+                )
+            )
+            for m in memberships
+        ]
+    )
+    return self.Update(['membership_specs'], self.messages.Feature(
+        membershipSpecs=membership_specs
+    ))
 
 
 class DisableCommand(UpdateCommandMixin, calliope_base.DeleteCommand):
@@ -247,7 +290,7 @@ class DisableCommand(UpdateCommandMixin, calliope_base.DeleteCommand):
       action='store_true',
       help=(
           'Disable the [fleet-default membership configuration]('
-          'https://cloud.google.com/kubernetes-engine/fleet-management/docs/manage-features).'
+          'https://docs.cloud.google.com/kubernetes-engine/fleet-management/docs/manage-features).'
           ' Does not change existing membership configurations.'
           ' Does nothing if the feature is disabled.'
       ),
